#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "logging.h"

#include "grabfile.h"
#include "grabfile_plain.h"

#ifdef CONFIG_USE_ZLIB
#include "grabfile_zlib.h"
#endif

#ifdef CONFIG_USE_LZMA
#include "grabfile_lzma.h"
#endif

static int compress_type(const char *filename)
{
	int fd;
	unsigned char buf[6];

	if ((fd = open(filename, O_RDONLY)) == -1)
		return -1;

	if (read(fd, buf, sizeof(buf)) == -1) {
		close(fd);
		return -1;
	}

	close(fd);

	if (buf[0] == 0xFD && buf[1] == '7' && buf[2] == 'z' && buf[3] == 'X' && buf[4] == 'Z' && buf[5] == 0x00)
		return LZMA_FILE;

	if (buf[0] == 0x1F && buf[1] == 0x8B)
		return GZIP_FILE;

	return PLAIN_FILE;
}

int grab_file(const char *filename, struct grab_data *fdata)
{
	switch (compress_type(filename)) {
#ifdef CONFIG_USE_ZLIB
		case GZIP_FILE:
			return gzip_grab_file(filename, fdata);
#endif
#ifdef CONFIG_USE_LZMA
		case LZMA_FILE:
			return lzma_grab_file(filename, fdata);
#endif
		case PLAIN_FILE:
			return plain_grab_file(filename, fdata);
	}
	fatal("Unknown compression type\n");
	return -1;
}

void release_file(struct grab_data *fdata)
{
	switch (fdata->type) {
#ifdef CONFIG_USE_ZLIB
		case GZIP_FILE:
			gzip_release_file(fdata);
			return;
#endif
#ifdef CONFIG_USE_LZMA
		case LZMA_FILE:
			lzma_release_file(fdata);
			return;
#endif
		case PLAIN_FILE:
			plain_release_file(fdata);
			return;
	}
	fatal("Unknown compression type\n");
}
