#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "grabfile_lzma.h"
#include "logging.h"
#include "testing.h"

#include <lzma.h>

#define IN_BUF_MAX 1024
#define OUT_BUF_MAX 4096

int lzma_grab_file(const char *filename, struct grab_data *fdata)
{
	int fd;
	unsigned char *buffer = NULL;
	unsigned char buf[IN_BUF_MAX];
	size_t len;
	lzma_ret ret;
	lzma_stream strm = LZMA_STREAM_INIT;

	fdata->data = NULL;
	fdata->size = 0;

	if ((fd = open(filename, O_RDONLY)) == -1) {
		error("open: %s\n", strerror(errno));
		return -1;
	}

	if ((ret = lzma_auto_decoder(&strm, UINT64_MAX, 0)) != LZMA_OK)
		fatal("lzma_auto_decoder: return code = %d\n", (int) ret);

	do {
		if ((len = read(fd, &buf, IN_BUF_MAX)) == -1) {
			if (errno == EINTR)
				continue;
			fatal("read: %s\n", strerror(errno));
		}

		strm.next_in  = buf;
		strm.avail_in = len;

		do {
			buffer = NOFAIL(realloc(buffer, strm.total_out + OUT_BUF_MAX));

			strm.next_out  = buffer + strm.total_out;
			strm.avail_out = OUT_BUF_MAX;

			ret = lzma_code(&strm, (len == 0 ? LZMA_FINISH : LZMA_RUN));

			if (ret != LZMA_OK && ret != LZMA_STREAM_END)
				fatal("lzma_code: return code = %d\n", (int) ret);

		} while (ret == LZMA_OK && strm.avail_in > 0);

	} while (len > 0);

	lzma_end(&strm);

	fdata->type = LZMA_FILE;
	fdata->data = buffer;
	fdata->size = strm.total_out;

	close(fd);
	return 0;
}

void lzma_release_file(struct grab_data *fdata)
{
	free(fdata->data);
}
