/* Support for compressed modules.  Willy Tarreau <willy@meta-x.org>
 * did the support for modutils, Andrey Borzenkov <arvidjaar@mail.ru>
 * ported it to module-init-tools, and I said it was too ugly to live
 * and rewrote it 8).
 *
 * (C) 2003 Rusty Russell, IBM Corporation.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

#include "grabfile_zlib.h"
#include "logging.h"
#include "testing.h"

#include <zlib.h>

static int grab_contents(gzFile *gzfd, struct grab_data *fdata)
{
	unsigned int max = 16384;
	int ret;

	fdata->data = NOFAIL(malloc(max));
	fdata->size = 0;

	while ((ret = gzread(gzfd, fdata->data + fdata->size, max - fdata->size)) > 0) {
		fdata->size += ret;
		if (fdata->size == max)
			fdata->data = NOFAIL(realloc(fdata->data, max *= 2));
	}
	if (ret < 0) {
		free(fdata->data);
		fdata->data = NULL;
		return -1;
	}

	return 0;
}

int gzip_grab_file(const char *filename, struct grab_data *fdata)
{
	gzFile gzfd;

	errno = 0;
	gzfd = gzopen(filename, "rb");
	if (!gzfd) {
		if (errno == ENOMEM)
			fatal("Memory allocation failure in gzopen\n");
		return -1;
	}
	if (grab_contents(gzfd, fdata) < 0) {
		gzclose(gzfd);
		return -1;
	}
	fdata->type = GZIP_FILE;
	gzclose(gzfd);
	return 0;
}

void gzip_release_file(struct grab_data *fdata)
{
	free(fdata->data);
}
